rm(list = ls()) # clear R memory

# select any file from the folder with the downloaded data and source files
fileName <- file.choose()

# set the working directory to the directory with the file
setwd (dirname(fileName)) 

############
# load data 
############

# Load tree data: 
# Alwyn Gentry dataset (Phillips et al. 2002. Missouri Botanical Press Louis, MO.)
# includes tree community data for 226 locations across the globe 

trees <- read.csv(file.choose()) # select "Gentry_stems.csv" downloaded using rdataretriever: https://retriever.readthedocs.io/en/latest/rdataretriever.html
trees <- trees[c(4, 6)] # select columns of interest
names(trees)[c(1,2)] <- c("community","size") # rename columns

# column 'size' is diameter at breast height (cm)
# column 'community' is the unique location (n = 226)
# Note: in the paper we used 187 (of the original 226) Gentry sites, where individual stems were surveyed for a standardized area (2 x 50 m transects = 0.1 ha per site)
# the full Gentry dataset can be downladed using rdataretriever 
#after data retriever has been installed:https://www.data-retriever.org/

trees$size <- trees$size^(8/3) # convert diameter at breast height to volume/mass (see paper for further details)
trees <- subset(trees, size !="0") # remove zeros from dataset (present for example in SIERRARO dataset)

# Load phytoplankton data: 
phytoplankton <- read.csv(file.choose()) # select "TRYNITROP data.csv" 
# Phytoplankton community data from 17 Atlantic stations (TRINITROP dataset; Huete-Ortega et al. 2012. Proceedings of the Royal Society of London B: Biological Sciences, 279(1734), pp.1815-1823.)
# column 'size' is individual cell biovolume (um3)
# column 'community' is the unique station (n = 17)

##########################
## Packages and libraries                              
##########################

# functions by Aaron Clauset & Laurent Dubroca: http://tuvalu.santafe.edu/~aaronc/powerlaws/plfit.r
source("plfit.r") 
# Note: 
# on line 206, nosmall error rate has been changed to 0.05 (5%) instead of 0.1 
# on lines 218-220, code has been modified to select the first value of xmin within the 25th percentile of the optmisied xmin

# functions by Cosma Shalizi: http://tuvalu.santafe.edu/~aaronc/powerlaws/Rcode_README.txt
source("pareto.r")
source("powerexp.r")
source("exp.r")

# functions by Edwards et al. 2016 https://github.com/andrew-edwards/fitting-size-spectra/blob/master/code/PLBfunctions.r
source("PLBfunctions.r") 

#############################
# select dataset for analysis:
#############################

df <- trees 
df <- phytoplankton

##########################################################################################
## decide to fit distributions to all data or data range where powerlaw is first optimised
##########################################################################################

useImposedXmin = F # T = fit distribution to all data F = data range where powerlaw is first optimised 

###############################################
### create empty dataframe for summary output 
###############################################

s.out <- data.frame(community=unique(df$community), imposedXmin = NA,
                    plfitExponent=NA, plfitXmin=NA, plfitD=NA, plfitLL=NA,
                    MLBExponent=NA, MLBXmin=NA, MLBXmax=NA,MLB.minNegLL=NA, 
                    powerExpExponent=NA, powerExpRate=NA, powerExpXmin=NA, powerExpLoglike=NA)

#################################
#### Fit power law distributions                        
##################################

# loop distrbution fitting over each community (forest transect or marine station)
uniq <- unique(unlist(df$community))

  for (iii in 1:length(uniq)){
    print(paste("running ", iii, " of ", length(uniq)))
    x <- subset(df, community == uniq[iii])
    myData <- x$size
    
    s.out$imposedXmin[iii] <- min(myData) # minimum body size in community 
    
    ## plfit from plfit.r
    # The function plfit estimates both xmin and alpha. It uses the likelihood method for alpha, 
    # trying various possible values of xmin. It then chooses the value of xmin that gives the lower KS 
    # value. The function automatically chooses the most appropriate method (discrete or continuous data)
    
    
    if (useImposedXmin == FALSE) {
      fitWithPlfit <- plfit(myData, nosmall=TRUE)
    }
    else {
      
      fitWithPlfit <- plfit(myData, "fixed", imposedXmin)
      
    }
    
    
    ## pareto.fit from pareto.r
    # fit pareto with the function pareto.fit, in the file pareto.R by Cosma Shalizi
    # the function pareto.fit actually calls the appropriate method 
    # (by default based on likelihood, but if otherwise specified based on regression)
    # pareto.fit.ml. Notice that in this case xmin is given in input (it is the threshold value)
    
    # use pareto.fit to get log-likelihood as plfit gives D
    tempFit <- pareto.fit(data=myData, threshold=fitWithPlfit$xmin, method="ml") 
    fitWithPlfit$loglike <- tempFit$loglike
    
    # summary output for power-law distribution
    s.out$plfitExponent[iii] <- -fitWithPlfit$alpha + 1
    s.out$plfitXmin[iii] <- fitWithPlfit$xmin
    s.out$plfitD[iii] <- fitWithPlfit$D
    s.out$plfitLL[iii] <- fitWithPlfit$loglike
    
    
    # Bounded MLE method from PLBfunctions.r
    # This method assumes that data follow a power law only within a range xmin - xmax and fits the
    # slope of the power law based on likelihood. The estimates for xmin and xmax match the values of
    # xmin and xmax in the data
    
    sub.x <- myData[myData >= fitWithPlfit$xmin]
    sum.log.x <- sum(log(sub.x))
    
    # Use analytical value of MLE b for PL model (Box 1, Edwards et al. 2007)
    #  as a starting point for nlm for MLE of b for PLB model.
    PL.bMLE <- 1/( log(min(sub.x)) - sum.log.x/length(sub.x)) - 1
    fitWithBoundedMLE <-  nlm(negLL.PLB, p=PL.bMLE, x=sub.x, n=length(sub.x), 
                              xmin=min(sub.x), xmax=max(sub.x), sumlogx=sum.log.x) #, print.level=2 )
    
    fitWithBoundedMLE$xmin <- min(sub.x)
    fitWithBoundedMLE$xmax <- max(sub.x)
    
    # summary output for bounded power-law distribution
    s.out$MLBExponent[iii] <- fitWithBoundedMLE$estimate + 1
    s.out$MLBXmin[iii] <- fitWithBoundedMLE$xmin
    s.out$MLBXmax[iii] <- fitWithBoundedMLE$xmax
    s.out$MLB.minNegLL[iii] <- fitWithBoundedMLE$minimum
    
    
    # power-exponential distribution using 'powerpexp.r' and 'exp.r'
    fitWithPowerExp <- tryCatch(
      {
        powerexp.fit(myData,threshold=fitWithPlfit$xmin)
      },
      error=function(cond)
      {
        message("powerexp.fit did not work")
        return(list(exponent=NA, rate=NA, xmin=NA, loglike=NA))
      })
    
    # summary output for power-exponential distribution
    s.out$powerExpExponent[iii] <- -fitWithPowerExp$exponent + 1
    s.out$powerExpRate[iii] <- fitWithPowerExp$rate
    s.out$powerExpXmin[iii] <- fitWithPowerExp$xmin
    s.out$powerExpLoglike[iii] <- fitWithPowerExp$loglike

    
    #############
    ## Plot data                                           
    ##############
    
    # plot rank/frequency style plot:

    usedData <- myData[myData >= fitWithPlfit$xmin] # select all x values for plotting

    plot(sort(myData, decreasing=TRUE), 1:length(myData), log="xy", cex.lab = 1.5,cex.axis = 1.2,cex.main = 2,
        xlab=expression(paste("Body size, ", italic(M))),
         ylab=expression( paste("Number of values ≥ ", italic(M))));
    
    xPLB <- sort(usedData, decreasing=TRUE) # select x values for optimised PL fit
    yPLB <- (1.0 - pPLB(x = xPLB, b = fitWithBoundedMLE$estimate, xmin = min(xPLB),
                        xmax = max(xPLB))) * length(usedData)
    
    try(
      yPEXP <- (1.0 - ppowerexp(xPLB, fitWithPlfit$xmin, fitWithPowerExp$exponent,fitWithPowerExp$rate)) * length(usedData)
    )
    
    yPL <- (1.0 - ppareto(xPLB, min(sub.x), fitWithPlfit$alpha)) * length(usedData)
    

    lines(xPLB, yPL, col="green", lwd = 2) # Power law = green
    lines(xPLB, yPLB, col="red", lwd = 2) # Bounded power law = red
    try(
      lines(xPLB, yPEXP, col="blue", lwd = 2) # Power exponential = blue
    )
  
    inset <- c(0.01, 0.01)
    legend("bottomleft", legend = c(paste("PL exp. + 1 =", signif(-fitWithPlfit$alpha + 1, 2)), paste("BPL exp. + 1 =", signif(fitWithBoundedMLE$estimate + 1, 2)), paste("PEXP exp. + 1 =", signif(-fitWithPowerExp$exponent + 1, 2))), col=c("green", "blue", "red"),lty=1:3, inset=inset)
    title(uniq[iii])
    
  }


head(s.out) # inspect output table

